@@ -19,3 +19,6 @@ uname = platform.uname()
 machine = platform.machine()
 arch = platform.architecture()[0]
 kernel = LooseVersion(uname[2]).version[:3]
+
+data_plugins_pkgs = []
+data_plugins_path = []
