@@ -16,10 +16,10 @@ IPDB vs. IPRoute
 ----------------
 
 These two modules, IPRoute and IPDB, use completely different
-approaches. The first one, IPRoute, just forward requests to
+approaches. The first one, IPRoute, just forwards requests to
 the kernel, and doesn't wait for the system state. So it's up
 to developer to check, whether the requested object is really
-set up, or not yet.
+set up or not.
 
 The latter, IPDB, is an asynchronously updated database, that
 starts several additional threads by default. If your
@@ -64,6 +64,58 @@ So, IPRoute is much simpler when you need to make a call and
 then exit, while IPDB is cheaper in terms of CPU performance
 if you implement a long-running program like a daemon.
 
+IPDB and other software
+-----------------------
+
+IPDB is designed to be a non-exclusive network settings database.
+There may be several IPDB instances on the same OS, as well as
+other network management software, such as NetworkManager etc.
+
+The IPDB transactions should not interfere with other software
+settings, unless they touch the same objects. E.g., if IPDB
+brings an interface up, while NM shuts it down, there will be
+a race condition.
+
+An example::
+
+    # IPDB code                       #  NetworkManager at the same time:
+    ipdb.interfaces['eth0'].up()      #
+    ipdb.interfaces['eth0'].commit()  #  $ sudo nmcli con down eth0
+    # ---> <---
+    # The eth0 state here is undefined. Some of the commands
+    # above will fail
+
+But as long as the software doesn't touch the same objects, there
+will be no conflicts. Another example::
+
+    # IPDB code                         # At the same time, NetworkManager
+    with ipdb.interfaces['eth0'] as i:  # adds addresses:
+        i.add_ip('172.16.254.2/24')     #  * 10.0.0.2/24
+        i.add_ip('172.16.254.3/24')     #  * 10.0.0.3/24
+    # ---> <---
+    # At this point the eth0 interface will have all four addresses.
+    # If the IPDB transaction fails by some reason, only IPDB addresses
+    # will be rolled back.
+
+There may be a need to prevent other software from changing the network
+settings. There is no locking at the kernel level, but IPDB can revert
+all the changes as soon as they appear on the interface::
+
+    # IPDB code
+    ipdb.interfaces['eth0'].freeze()
+                                       # Here some other software tries to
+                                       # add an address, or to remove the old
+                                       # one
+    # ---> <---
+    # At this point the eth0 interface will have all the same settings as
+    # at the `freeze()` call moment. Newly added addresses will be removed,
+    # all the deleted addresses will be restored.
+    #
+    # Please notice, that an address removal may cause also a routes removal,
+    # and that is the thing that IPDB can not neither prevent, nor revert.
+
+    ipdb.interfaces['eth0'].unfreeze()
+
 Quickstart
 ----------
 
@@ -172,6 +224,23 @@ managers in the same way as IPDB does itself::
 On exit, the context manager will authomatically `commit()`
 the transaction.
 
+Read-only interface views
+-------------------------
+
+Using an interface as a context manager **will** start a
+transaction. Sometimes it is not what one needs. To avoid
+unnecessary transactions, and to avoid the risk to occasionally
+change interface attributes, one can use read-only views::
+
+    with ipdb.interfaces[1].ro as iface:
+        print(iface.ifname)
+        print(iface.address)
+
+The `.ro` view neither starts transactions, nor allows to
+change anything, raising the `RuntimeError` exception.
+
+The same read-only views are available for routes and rules.
+
 Create interfaces
 -----------------
 
@@ -187,6 +256,73 @@ The `IPDB.create()` call has the same syntax as
 `IPRoute.link('add', ...)`, except you shouldn't specify
 the `'add'` command. Refer to `IPRoute` docs for details.
 
+Please notice, that the interface object stays in the database
+even if there was an error during the interface creation. It is
+done so to make it possible to fix the interface object and try
+to run `commit()` again. Or you can drop the interface object
+with the `.remove().commit()` call.
+
+IP address management
+---------------------
+
+IP addresses on interfaces may be managed using `add_ip()` and
+`del_ip()`::
+
+    with ipdb.interfaces['eth0'] as eth:
+        eth.add_ip('10.0.0.1/24')
+        eth.add_ip('10.0.0.2/24')
+        eth.add_ip('2001:4c8:1023:108::39/64')
+        eth.del_ip('172.16.12.5/24')
+
+The address format may be either a string with `'address/mask'`
+notation, or a pair of `'address', mask`::
+
+    with ipdb.interfaces['eth0'] as eth:
+        eth.add_ip('10.0.0.1', 24)
+        eth.del_ip('172.16.12.5', 24)
+
+The `ipaddr` attribute contains all the IP addresses of the
+interface, which are acessible in different ways. Getting an
+iterator from `ipaddr` gives you a sequence of tuples
+`('address', mask)`::
+
+    >>> for addr in ipdb.interfaces['eth0'].ipaddr:
+    ...    print(ipaddr)
+    ...
+    ('10.0.0.2', 24)
+    ('10.0.0.1', 24)
+
+Getting one IP from `ipaddr` returns a dict object with full spec:
+
+    >>> ipdb.interfaces['eth0'].ipaddr[0]:
+        {'family': 2,
+         'broadcast': None,
+         'flags': 128,
+         'address': '10.0.0.2',
+         'prefixlen': 24,
+         'local': '10.0.0.2'}
+
+    >>> ipdb.intefaces['eth0'].ipaddr['10.0.0.2/24']:
+        {'family': 2,
+         'broadcast': None,
+         'flags': 128,
+         'address': '10.0.0.2',
+         'prefixlen': 24,
+         'local': '10.0.0.2'}
+
+The API is a bit weird, but it's because of historical reasons. In
+the future it may be changed.
+
+Another feature of the `ipaddr` attribute is views::
+
+    >>> ipdb.interfaces['eth0'].ipaddr.ipv4:
+        (('10.0.0.2', 24), ('10.0.0.1', 24))
+    >>> ipdb.interfaces['eth0'].ipaddr.ipv6:
+        (('2001:4c8:1023:108::39', 64),)
+
+The views, as well as the `ipaddr` attribute itself are not supposed
+to be changed by user, but only by the internal API.
+
 Bridge interfaces
 -----------------
 
@@ -197,10 +333,21 @@ time etc. Names of these properties start with `br_`, like
 
     [x for x in dir(ipdb.interfaces.virbr0) if x.startswith('br_')]
 
+Bridge ports
+------------
+
+IPDB supports specific bridge port parameters, such as proxyarp,
+unicast/multicast flood, cost etc.::
+
+    with ipdb.interfaces['br-port0'] as p:
+        p.brport_cost = 200
+        p.brport_unicast_flood = 0
+        p.brport_proxyarp = 0
+
 Ports management
 ----------------
 
-IPDB provides a uniform API to manage ports::
+IPDB provides a uniform API to manage bridge, bond and vrf ports::
 
     with ipdb.interfaces['br-int'] as br:
         br.add_port('veth0')
@@ -214,8 +361,6 @@ Both `add_port()` and `del_port()` accept three types of arguments:
     * `ipdb.interfaces.veth1` -- IPDB interface object
     * `700` -- interface index, an integer
 
-The same methods are used to manage bridge, bond and vrf ports.
-
 Routes management
 -----------------
 
@@ -261,7 +406,7 @@ follows::
         route.metrics.mtu = 1500
 
     # access the default route
-    print(ipdb.routes['default])
+    print(ipdb.routes['default'])
 
     # change the default gateway
     with ipdb.routes['default'] as route:
@@ -523,21 +668,28 @@ after some delay.
 The class API
 -------------
 '''
+import sys
 import atexit
 import logging
 import traceback
 import threading
 
+from functools import partial
+from pprint import pprint
 from pyroute2 import config
 from pyroute2.common import uuid32
+from pyroute2.common import basestring
 from pyroute2.iproute import IPRoute
 from pyroute2.netlink.rtnl import RTM_GETLINK, RTNL_GROUPS
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
-from pyroute2.ipdb import rule
-from pyroute2.ipdb import route
-from pyroute2.ipdb import interface
-from pyroute2.ipdb.linkedset import IPaddrSet, SortedIPaddrSet
+from pyroute2.ipdb import rules
+from pyroute2.ipdb import routes
+from pyroute2.ipdb import interfaces
+from pyroute2.ipdb.routes import BaseRoute
 from pyroute2.ipdb.transactional import SYNC_TIMEOUT
+from pyroute2.ipdb.linkedset import IPaddrSet
+from pyroute2.ipdb.linkedset import SortedIPaddrSet
+from pyroute2.ipdb.utils import test_reachable_icmp
 
 log = logging.getLogger(__name__)
 
@@ -583,11 +735,17 @@ class IPDB(object):
                  restart_on_error=None, nl_async=None,
                  nl_bind_groups=RTNL_GROUPS,
                  ignore_rtables=None, callbacks=None,
-                 sort_addresses=False):
+                 sort_addresses=False, plugins=None):
+        plugins = plugins or ['interfaces', 'routes', 'rules']
+        pmap = {'interfaces': interfaces,
+                'routes': routes,
+                'rules': rules}
         self.mode = mode
-        self.sort_addresses = sort_addresses
+        self._stdout = sys.stdout
+        self._ipaddr_set = SortedIPaddrSet if sort_addresses else IPaddrSet
         self._event_map = {}
         self._deferred = {}
+        self._ensure = []
         self._loaded = set()
         self._mthread = None
         self._nl_own = nl is None
@@ -595,7 +753,7 @@ class IPDB(object):
         self.mnl = None
         self.nl = nl
         self.nl_bind_groups = nl_bind_groups
-        self._plugins = [interface, route, rule]
+        self._plugins = [pmap[x] for x in plugins if x in pmap]
         if isinstance(ignore_rtables, int):
             self._ignore_rtables = [ignore_rtables, ]
         elif isinstance(ignore_rtables, (list, tuple, set)):
@@ -637,9 +795,18 @@ class IPDB(object):
         # init the database
         self.initdb()
 
+        # init the dir() cache
+        self.__dir_cache__ = [i for i in self.__class__.__dict__.keys()
+                              if i[0] != '_']
+        self.__dir_cache__.remove('serve_forever')
+        self.__dir_cache__.extend(list(self._deferred.keys()))
+
         #
         atexit.register(self.release)
 
+    def __dir__(self):
+        return self.__dir_cache__
+
     def __enter__(self):
         return self
 
@@ -874,26 +1041,127 @@ class IPDB(object):
     def create(self, kind, ifname, reuse=False, **kwarg):
         return self.interfaces.add(kind, ifname, reuse, **kwarg)
 
+    def ensure(self, cmd='add', reachable=None, condition=None):
+        if cmd == 'reset':
+            self._ensure = []
+        elif cmd == 'run':
+            for f in self._ensure:
+                f()
+        elif cmd == 'add':
+            if isinstance(reachable, basestring):
+                reachable = reachable.split(':')
+                if len(reachable) == 1:
+                    f = partial(test_reachable_icmp, reachable[0])
+                else:
+                    raise NotImplementedError()
+                self._ensure.append(f)
+            else:
+                if sys.stdin.isatty():
+                    pprint(self._ensure, stream=self._stdout)
+        elif cmd == 'print':
+            pprint(self._ensure, stream=self._stdout)
+        elif cmd == 'get':
+            return self._ensure
+        else:
+            raise NotImplementedError()
+
+    def review(self):
+        ilist = {}
+        rlist = {}
+        ret = {}
+
+        # collect interface transactions
+        for ifname in getattr(self, 'by_name', {}):
+            try:
+                ilist[ifname] = self.interfaces[ifname].review()
+            except TypeError:
+                pass
+        # collect route transactions
+        for table in getattr(getattr(self, 'routes', None),
+                             'tables', {}):
+            for key in self.routes.tables[table].keys():
+                try:
+                    rlist[ifname] = self.routes.tables[table][key].review()
+                except TypeError:
+                    pass
+
+        if ilist:
+            ret['interfaces'] = ilist
+        if rlist:
+            ret['routes'] = rlist
+
+        if not ret:
+            raise TypeError('no transactions started')
+        return ret
+
+    def drop(self):
+        for (system, tx) in self.review().items():
+            for key in tx:
+                getattr(self, system)[key].drop()
+
     def commit(self, transactions=None, phase=1):
         # what to commit: either from transactions argument, or from
         # started transactions on existing objects
         if transactions is None:
             # collect interface transactions
             txlist = [(x, x.current_tx) for x
-                      in self.by_name.values() if x.local_tx.values()]
+                      in getattr(self, 'by_name', {}).values()
+                      if x.local_tx.values()]
             # collect route transactions
-            for table in self.routes.tables.keys():
+            for table in getattr(getattr(self, 'routes', None),
+                                 'tables', {}).keys():
                 txlist.extend([(x, x.current_tx) for x in
                                self.routes.tables[table]
                                if x.local_tx.values()])
-            txlist = sorted(txlist,
-                            key=lambda x: x[1]['ipdb_priority'],
-                            reverse=True)
             transactions = txlist
 
         snapshots = []
         removed = []
 
+        tx_ipdb_prio = []
+        tx_main = []
+        tx_prio1 = []
+        tx_prio2 = []
+        tx_prio3 = []
+        for (target, tx) in transactions:
+            # 8<------------------------------
+            # first -- explicit priorities
+            if tx['ipdb_priority']:
+                tx_ipdb_prio.append((target, tx))
+                continue
+            # 8<------------------------------
+            # routes
+            if isinstance(target, BaseRoute):
+                tx_prio3.append((target, tx))
+                continue
+            # 8<------------------------------
+            # intefaces
+            kind = target.get('kind', None)
+            if kind in ('vlan', 'vxlan', 'gre', 'tuntap', 'vti', 'vrf'):
+                tx_prio1.append((target, tx))
+            elif kind in ('bridge', 'bond'):
+                tx_prio2.append((target, tx))
+            else:
+                tx_main.append((target, tx))
+            # 8<------------------------------
+
+        # explicitly sorted transactions
+        tx_ipdb_prio = sorted(tx_ipdb_prio,
+                              key=lambda x: x[1]['ipdb_priority'],
+                              reverse=True)
+
+        # FIXME: this should be documented
+        #
+        # The final transactions order:
+        # 1. any txs with ipdb_priority (sorted by that field)
+        #
+        # Then come default priorities (no ipdb_priority specified):
+        # 2. all the rest
+        # 3. vlan, vxlan, gre, tuntap, vti, vrf
+        # 4. bridge, bond
+        # 5. routes
+        transactions = tx_ipdb_prio + tx_main + tx_prio1 + tx_prio2 + tx_prio3
+
         try:
             for (target, tx) in transactions:
                 if target['ipdb_scope'] == 'detached':
@@ -904,13 +1172,21 @@ class IPDB(object):
                 if phase == 1:
                     s = (target, target.pick(detached=True))
                     snapshots.append(s)
+                # apply the changes, but NO rollback -- only phase 1
                 target.commit(transaction=tx,
                               commit_phase=phase,
                               commit_mask=phase)
+                # if the commit above fails, the next code
+                # branch will run rollbacks
         except Exception:
             if phase == 1:
+                # run rollbacks for ALL the collected transactions,
+                # even successful ones
                 self.fallen = transactions
-                self.commit(transactions=snapshots, phase=2)
+                txs = filter(lambda x: not ('create' ==
+                                            x[0]['ipdb_scope'] ==
+                                            x[1]['ipdb_scope']), snapshots)
+                self.commit(transactions=txs, phase=2)
             raise
         else:
             if phase == 1:
@@ -997,9 +1273,3 @@ class IPDB(object):
                                 pass
                             if len(self._cb_threads.get(cuid, ())) == 0:
                                 del self._cb_threads[cuid]
-
-    def init_ipaddr_set(self):
-        if self.sort_addresses:
-            return SortedIPaddrSet()
-        else:
-            return IPaddrSet()
