@@ -97,7 +97,6 @@ from socket import SO_RCVBUF
 from socket import SO_SNDBUF
 
 from pyroute2 import config
-from pyroute2.config import SocketBase
 from pyroute2.common import AddrPool
 from pyroute2.common import DEFAULT_RCVBUF
 from pyroute2.netlink import nlmsg
@@ -869,10 +868,10 @@ class NetlinkSocket(NetlinkMixin):
         with self.lock:
             if self._sock is not None:
                 self._sock.close()
-            self._sock = SocketBase(AF_NETLINK,
-                                    SOCK_DGRAM,
-                                    self.family,
-                                    self._fileno)
+            self._sock = config.SocketBase(AF_NETLINK,
+                                           SOCK_DGRAM,
+                                           self.family,
+                                           self._fileno)
             for name in ('getsockname', 'getsockopt', 'makefile',
                          'setsockopt', 'setblocking', 'settimeout',
                          'gettimeout', 'shutdown', 'recvfrom',
@@ -894,7 +893,7 @@ class NetlinkSocket(NetlinkMixin):
                 def patch(data, bsize):
                     data[0:] = self._sock.recv(bsize)
                 self._sock.recv_into = patch
-            self.setsockopt(SOL_SOCKET, SO_SNDBUF, 32768)
+            self.setsockopt(SOL_SOCKET, SO_SNDBUF, 1024 * 1024)
             self.setsockopt(SOL_SOCKET, SO_RCVBUF, 1024 * 1024)
             if self.all_ns:
                 self.setsockopt(SOL_NETLINK, NETLINK_LISTEN_ALL_NSID, 1)
